## Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002, 2003, 2005, 2006,
##               2007 John W. Eaton
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or (at
## your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {Function File} {[@var{p}, @var{s}] =} polyfit (@var{x}, @var{y}, @var{n})
## Return the coefficients of a polynomial @var{p}(@var{x}) of degree
## @var{n} that minimizes
## @iftex
## @tex
## $$
## \sum_{i=1}^N (p(x_i) - y_i)^2
## $$
## @end tex
## @end iftex
## @ifinfo
## @code{sumsq (p(x(i)) - y(i))},
## @end ifinfo
##  to best fit the data in the least squares sense.
##
## The polynomial coefficients are returned in a row vector.
##
## If two output arguments are requested, the second is a structure
## containing the following fields:
##
## @table @code
## @item R
## The Cholesky factor of the Vandermonde matrix used to compute the
## polynomial coefficients.
## @item X
## The Vandermonde matrix used to compute the polynomial coefficients.
## @item df
## The degrees of freedom.
## @item normr
## The norm of the residuals.
## @item yf
## The values of the polynomial for each value of @var{x}.
## @end table
## 
## If a fourth bool parameter @var{scale} is given and set to
## @code{true}, the values of @var{x} are scaled by their maximum absolute
## value before doing the actual calculation. This helps in numercial difficult
## cases. The returned coefficients @var{p} are valid for the unscaled
## values of @var{x}, while the values in the structure @var{S} are
## valid for the scaled polynomial used for the actual calculation.
## @end deftypefn

## Author: KH <Kurt.Hornik@wu-wien.ac.at>
## Created: 13 December 1994
## Adapted-By: jwe

function [p, s, mu] = polyfit (x, y, n, scale)

  if (nargin < 3 || nargin > 4)
    print_usage ();
  endif

  if (nargin == 3)
    scale = false;
  else
    if ( ! isbool(scale) || ! isscalar(scale))
      error ("polyfit: scale must be a bool scalar");
    endif
  endif

  if (! (isvector (x) && isvector (y) && size_equal (x, y)))
    error ("polyfit: x and y must be vectors of the same size");
  endif

  if (! (isscalar (n) && n >= 0 && ! isinf (n) && n == round (n)))
    error ("polyfit: n must be a nonnegative integer");
  endif

  y_is_row_vector = (rows (y) == 1);

  l = length (x);
  x = reshape (x, l, 1);
  y = reshape (y, l, 1);
  ## FIXME: is reshape() faster than (:) ?

  ## scale the x values
  if (scale)
    xmax = max(abs(x));
    x = x/xmax;
  endif

  X = (x * ones (1, n+1)) .^ (ones (l, 1) * (n : -1 : 0));
  # FIXME: maybe use repmat() for the ones() part, but this here seems faster

  p = X \ y;

  if (nargout > 1)

    yf = X*p;

    if (y_is_row_vector)
      s.yf = yf.';
    else
      s.yf = yf;
    endif

    [s.R, dummy] = chol (X'*X);
    ## FIXME: what if dummy is !0 ?
    s.X = X;
    s.df = l - n - 1;
    s.normr = norm (yf - y);

  endif

  ## Scale the polynomial back and return a row vector
  if (scale)
    power = length(p)-1:-1:0;
    power = (xmax .^ power)(:);
    p = (p ./ power);
  endif

  ## Return a row vector
  p = p(:)';
endfunction

%!test
%! x = [-2, -1, 0, 1, 2];
%! assert(all (all (abs (polyfit (x, x.^2+x+1, 2) - [1, 1, 1]) < sqrt (eps))));

%!test
%! x = [-2, -1, 0, 1, 2];
%! assert(all (all (abs (polyfit (x, x.^2+x+1, 3) - [0, 1, 1, 1]) < sqrt (eps))));

%!error polyfit ([1, 2; 3, 4], [1, 2; 3, 4], 4);

%!test
%! x = [-2, -1, 0, 1, 2];
%! fail("polyfit (x, x.^2+x+1)");

%!test
%! x = [-2, -1, 0, 1, 2];
%! fail("polyfit (x, x.^2+x+1, [])");

%% test difficult case where scaling is really needed
%% checks also usage of second output argument
%!shared s1, s2
%!  x = [ -1196.4, -1195.2, -1194, -1192.8, -1191.6, -1190.4, -1189.2, -1188, \
%!        -1186.8, -1185.6, -1184.4, -1183.2, -1182 ];
%!  y = [ 315571.7086, 315575.9618, 315579.4195, 315582.6206, 315585.4966,    \
%!        315588.3172, 315590.9326, 315593.5934, 315596.0455, 315598.4201,    \
%!        315600.7143, 315602.9508, 315605.1765 ];
%!  [p1,s1] = polyfit(x,y,10);
%!  [p2,s2] = polyfit(x,y,10, true);
%!
%!  # fails without scaling (s1.norm ~6.33)
%!error assert(s1.normr, 0.08,  0.01);
%!
%!  # but works with scaling
%!assert(s2.normr, 0.08,  0.01);

